package com.eksad.masterdata.service.invoicenjbnsc;

import com.eksad.ddms.common.dto.GatewayOrafinDTO;
import com.eksad.ddms.common.dto.GatewayPartOrafinDTO;
import com.eksad.ddms.common.dto.GatewayServiceOrafinDTO;
import com.eksad.ddms.common.dto.ResponseOrafinDTO;
import com.eksad.ddms.common.h2.dto.apigateway.request.PullNotaJBSCRequestDTO;
import com.eksad.ddms.common.h2.dto.ui.ResponseErrorDTO;
import com.eksad.ddms.common.uri.apigateway.APIGatewayURI;
import com.eksad.masterdata.common.RequestUtil;
import com.eksad.masterdata.controller.apidgi.invoicenjbnsc.DGIInvoiceNjbNscRESTController;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.netflix.hystrix.contrib.javanica.annotation.HystrixCommand;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpMethod;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;

import java.io.IOException;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.Duration;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Locale;
import java.util.logging.Level;
import java.util.logging.Logger;

@Service
public class DGIInvoiceNjbNscService {
    DateTimeFormatter dateTimeFormatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss", Locale.getDefault());

    private RestTemplate restTemplate = new RestTemplate();

    @Value("${dam.masterdata.h2}")
    private String md_h2_url;

    @Value("${dam.gigr.h3}")
    private String gigr_h3_url;

    @Value("${dam.custpurch.h3}")
    private String custpurch_h3_url;

    @Autowired
    private RequestUtil requestUtil;

    public ResponseEntity<Object> postRequestPullNJBNSC(PullNotaJBSCRequestDTO dto) throws ParseException {
        Object data = pullNJBNSC(dto);

        if(data != null){
            return ResponseEntity.status(HttpStatus.OK).body(data);
        }
        return null;
    }

    private Object pullNJBNSC(PullNotaJBSCRequestDTO dto) {
        ResponseEntity<Object> response = restTemplate.exchange(
                md_h2_url
                        + APIGatewayURI.PULL_NSC_NJB.getUri(),
                HttpMethod.POST,
                new HttpEntity<>(dto, requestUtil.getPreFormattedRequestWithToken().getHeaders()),
                Object.class);
        return response.getBody();
    }

    public void checkToTimeIs7Days(String fromTime, String toTime) throws Exception {
        DateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        Date from = dateFormat.parse(fromTime);
        Date to = dateFormat.parse(toTime);
        LocalDateTime fromLocal = from.toInstant().atZone(ZoneId.of("Asia/Jakarta")).toLocalDateTime();
        LocalDateTime toLocal = to.toInstant().atZone(ZoneId.of("Asia/Jakarta")).toLocalDateTime();
        Duration durasi = Duration.between(fromLocal, toLocal);
        long dayDifference = Math.abs(durasi.toHours());

        if (dayDifference > (7 * 24) + 1) {  // if duration greater than 24*7 + 1 hours
            throw new Exception("tenggat waktu hanya diperbolehkan 7 hari");
        }
    }

    public void exceptionHelper(String time, String format, String message) throws Exception {
        if (time == null || time.trim().isEmpty()) {
            throw new Exception(message + " tidak boleh kosong");
        }

        try {
            SimpleDateFormat dateFormat = new SimpleDateFormat(format);
            dateFormat.parse(time);
        } catch (ParseException e) {
            throw new Exception("format " + message.concat(" tidak sesuai, seharusnya ").concat(format));
        }

    }

    public void exceptionHelperNotMandatory(String time, String format, String message) throws Exception {
        if (time != null && !time.trim().isEmpty()) {
            try {
                SimpleDateFormat dateFormat = new SimpleDateFormat(format);
                dateFormat.parse(time);
            } catch (ParseException e) {
                throw new Exception("format " + message.concat(" tidak sesuai, seharusnya ").concat(format));
            }
        }
    }

    public boolean isNotValidDate(String input) {
        try {
            dateTimeFormatter.parse(input);
            return false;
        } catch (Exception e) {
            return true;
        }
    }

    public void dealerValidation(String dealerHeader, String dealerRequest) throws Exception {
        if (dealerHeader != null && dealerHeader != dealerRequest) {
            if (!dealerHeader.equalsIgnoreCase(dealerRequest)) {
                throw new Exception("gagal dealerId " + dealerRequest + " tidak sesuai");
            }
        }
    }

    @HystrixCommand(fallbackMethod = "defaultPostFallback")
    public ResponseEntity<Object> postWorkOrderOrafin(GatewayOrafinDTO dto, String dealerid) throws IOException {
        try {
            System.out.println(new ObjectMapper().writeValueAsString(dto));
            List<String> pass = new ArrayList<>();
            if (dto.getNoWorkOrder().contains("WO")) {
                //kalo wo njb wajib
                if (dto.getNoNJB() == null || dto.getNoNJB().isEmpty()) {
                    throw new Exception("No NJB Tidak Boleh Kosong, Mohon Periksa Kembali Data");
                }
                if (dto.getTanggalNJB() == null || dto.getTanggalNJB().isEmpty()) {
                    throw new Exception("Tanggal NJB Tidak Boleh Kosong, Mohon Periksa Kembali Data");
                }
                if (dto.getTotalHargaNJB() == null || dto.getTotalHargaNJB() == 0 || dto.getTotalHargaNJB().isNaN()) {
                    throw new Exception("Total Harga NJB Tidak Boleh Kosong, Mohon Periksa Kembali Data");
                }
                if (dto.getHondaIdSA() == null || dto.getHondaIdSA().isEmpty()) {
                    throw new Exception("Honda Id SA Tidak Boleh Kosong, Mohon Periksa Kembali Data");
                }
                if (dto.getHondaIdMekanik() == null || dto.getHondaIdMekanik().isEmpty()) {
                    throw new Exception("Honda Id Mekanik Tidak Boleh Kosong, Mohon Periksa Kembali Data");
                }
                if (dto.getNjb().isEmpty() || dto.getNjb() == null) {
                    throw new Exception("NJB Tidak Boleh Kosong, Mohon Periksa Kembali Data");
                }

                if (dto.getNoNSC() == null || dto.getNoNSC().isEmpty() && !dto.getTanggalNSC().isEmpty()) {
                    throw new Exception("No NSC harus dilengkapi");
                } else if (dto.getNoNSC() == null || dto.getNoNSC().isEmpty() && dto.getTotalHargaNSC() != 0) {
                    throw new Exception("NO NSC harus dilengkapi");
                } else if (dto.getNoNSC() == null || dto.getNoNSC().isEmpty() && !dto.getNsc().isEmpty()) {
                    throw new Exception("No NSC harus dilengkapi");
                } else if (!dto.getNoNSC().isEmpty() && dto.getTanggalNSC().isEmpty()) {
                    throw new Exception("Tanggal NSC harus dilengkapi");
                } else if (!dto.getNoNSC().isEmpty() && dto.getTotalHargaNSC() == 0) {
                    throw new Exception("Total Harga NSC harus dilengkapi");
                } else if (!dto.getNoNSC().isEmpty() && dto.getNsc().isEmpty()) {
                    throw new Exception("NSC harus dilengkapi");
                }
            }
            if (dto.getNoWorkOrder().contains("SO")) {
                //kalo so nsc wajib
                if (dto.getNoNSC() == null || dto.getNoNSC().isEmpty()) {
                    throw new Exception("No NSC Tidak Boleh Kosong, Mohon Periksa Kembali Data");
                }
                if (dto.getTanggalNSC() == null || dto.getTanggalNSC().isEmpty()) {
                    throw new Exception("Tanggal NSC Tidak Boleh Kosong, Mohon Periksa Kembali Data");
                }
                if (dto.getTotalHargaNSC() == null || dto.getTotalHargaNSC() == 0 || dto.getTotalHargaNSC().isNaN()) {
                    throw new Exception("Total Harga Tidak Boleh Kosong, Mohon Periksa Kembali Data");
                }

                if (dto.getNoNJB() == null || dto.getNoNJB().isEmpty() && !dto.getTanggalNJB().isEmpty()) {
                    throw new Exception("No NJB harus dilengkapi");
                } else if (dto.getNoNJB() == null || dto.getNoNJB().isEmpty() && dto.getTotalHargaNJB() != 0) {
                    throw new Exception("No NJB harus dilengkapi");
                } else if (dto.getNoNJB() == null || dto.getNoNJB().isEmpty() && !dto.getHondaIdMekanik().isEmpty()) {
                    throw new Exception("No NJB harus dilengkapi");
                } else if (dto.getNoNJB() == null || dto.getNoNJB().isEmpty() && !dto.getHondaIdSA().isEmpty()) {
                    throw new Exception("No NJB harus dilengkapi");
                } else if (dto.getNoNJB() == null || dto.getNoNJB().isEmpty() && !dto.getNjb().isEmpty()) {
                    throw new Exception("No NJB harus dilengkapi");
                } else if (!dto.getNoNJB().isEmpty() && dto.getTanggalNJB().isEmpty()) {
                    throw new Exception("Tanggal NJB harus dilengkapi");
                } else if (!dto.getNoNJB().isEmpty() && dto.getTotalHargaNJB() == 0) {
                    throw new Exception("Total Harga NJB harus dilengkapi");
                } else if (!dto.getNoNJB().isEmpty() && dto.getHondaIdMekanik().isEmpty()) {
                    throw new Exception("Honda Id Mekanik harus dilengkapi");
                } else if (!dto.getNoNJB().isEmpty() && dto.getHondaIdSA().isEmpty()) {
                    throw new Exception("Honda Id SA harus dilengkapi");
                } else if (!dto.getNoNJB().isEmpty() && dto.getNjb().isEmpty()) {
                    throw new Exception("NJB harus dilengkapi");
                }
            }
//        nullEmptyExceptionHelper(dto,pass);
            exceptionHelperNotMandatory(dto.getCreatedTime(), "dd/MM/yyyy HH:mm:ss", "createdTime");
            exceptionHelperNotMandatory(dto.getModifiedTime(), "dd/MM/yyyy HH:mm:ss", "modifiedTime");
            exceptionHelperNotMandatory(dto.getTanggalNJB(), "dd/MM/yyyy HH:mm:ss", "tanggalNJB");
            exceptionHelperNotMandatory(dto.getTanggalNSC(), "dd/MM/yyyy HH:mm:ss", "tanggalNSC");
            dealerValidation(dealerid, dto.getDealerId());
            if (!dto.getNjb().isEmpty() || dto.getNjb() != null) {
                for (GatewayServiceOrafinDTO njb : dto.getNjb()) {
                    exceptionHelperNotMandatory(njb.getCreatedTime(), "dd/MM/yyyy HH:mm:ss", "njb createdTime");
                    exceptionHelperNotMandatory(njb.getModifiedTime(), "dd/MM/yyyy HH:mm:ss", "njb modifiedTime");
                }
            }
            if (!dto.getNsc().isEmpty() || dto.getNsc() != null) {
                for (GatewayPartOrafinDTO nsc : dto.getNsc()) {
                    exceptionHelperNotMandatory(nsc.getCreatedTime(), "dd/MM/yyyy HH:mm:ss", "nsc createdTime");
                    exceptionHelperNotMandatory(nsc.getModifiedTime(), "dd/MM/yyyy HH:mm:ss", "nsc modifiedTime");
                }
            }
        } catch (Exception ex) {
            Logger.getLogger(DGIInvoiceNjbNscRESTController.class.getName()).log(Level.SEVERE, null, ex);
            ResponseErrorDTO res = new ResponseErrorDTO();
            res.setMessage(ex.getMessage());
            res.setStatus(HttpStatus.BAD_REQUEST.value());
            res.setError(HttpStatus.BAD_REQUEST.getReasonPhrase());
            return ResponseEntity.badRequest().body(res);
        }

        Object data = pushNJBNSC(dto);

        return ResponseEntity.status(HttpStatus.OK).body(data);
    }

    private Object pushNJBNSC(GatewayOrafinDTO dto) throws IOException {
        ObjectMapper obm = new ObjectMapper();
        ResponseEntity<String> response = restTemplate.exchange(
                md_h2_url
                        + APIGatewayURI.PUSH_NSC_NJB.getUri(),
                HttpMethod.POST,
                new HttpEntity<>(dto, requestUtil.getPreFormattedRequestWithToken().getHeaders()),
                String.class);
        ResponseOrafinDTO resOra = obm.readValue(response.getBody(), new TypeReference<ResponseOrafinDTO>() { });
        return resOra;
    }
}
